package com.nisum.producer;

import com.mongodb.spark.config.ReadConfig;

import java.util.concurrent.TimeoutException;

import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.functions;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.storage.StorageLevel;

public class SparkProducer {
     static String TOPIC = "UserInfoTopic";

    public static void main(String[] args) throws TimeoutException, StreamingQueryException {

        SparkSession spark = SparkSession.builder()
                .master("local")
                .appName("MongoSparkConnectorIntro")
                .config("spark.mongodb.input.uri", "mongodb://127.0.0.1/UserDB.users_info")
                .getOrCreate();

        ReadConfig readConfigUserInfo = ReadConfig.create(spark)
                .withOption("uri", "mongodb://localhost:27017")
                .withOption("database", "UserDB")
                .withOption("collection", "users_info");

        Dataset<Row> datasetUserInfo = spark.read()
                .format("mongo")
                .options(readConfigUserInfo.asOptions())
                .load();
        datasetUserInfo.createOrReplaceTempView("user");

        ReadConfig readConfigUserCardInfo = ReadConfig.create(spark)
                .withOption("uri", "mongodb://localhost:27017")
                .withOption("database", "UserDB")
                .withOption("collection", "users_cards_info");

        Dataset<Row> datasetUserCardInfo = spark.read()
                .format("mongo")
                .options(readConfigUserCardInfo.asOptions())
                .load();
        datasetUserCardInfo.createOrReplaceTempView("card");

        datasetUserInfo.show();
        datasetUserCardInfo.show();

        Dataset<Row> userCardInfo = spark
                .sql("SELECT user.gender,user.regMobileNo,user.userName,card.userId as userId,card._id as cardId,card.cardNumber,card.cardStatus,card.cardType FROM user JOIN card ON user._id = card.userId");
        userCardInfo.show();

        Dataset<Row> userCardInfoJson = userCardInfo.withColumn("value", functions.to_json(functions.struct(functions.col("gender"),
                functions.col("regMobileNo"), functions.col("userName"),
                functions.col("userId"), functions.col("cardId"),
                functions.col("cardNumber"), functions.col("cardStatus"),
                functions.col("cardType")))).alias("value").select("value").persist(StorageLevel.MEMORY_AND_DISK());
        userCardInfoJson.show(false);

        userCardInfoJson
                .write()
                .format("kafka")
                .option("kafka.bootstrap.servers", "localhost:9092")
                .option("topic", TOPIC).save();
        spark.close();

    }
}

